# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.create_email_provider_response_content import CreateEmailProviderResponseContent
from ...types.email_provider_credentials_schema import EmailProviderCredentialsSchema
from ...types.email_provider_name_enum import EmailProviderNameEnum
from ...types.email_specific_provider_settings_with_additional_properties import (
    EmailSpecificProviderSettingsWithAdditionalProperties,
)
from ...types.get_email_provider_response_content import GetEmailProviderResponseContent
from ...types.update_email_provider_response_content import UpdateEmailProviderResponseContent
from .raw_client import AsyncRawProviderClient, RawProviderClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ProviderClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawProviderClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawProviderClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawProviderClient
        """
        return self._raw_client

    def get(
        self,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetEmailProviderResponseContent:
        """
        Retrieve details of the [email provider configuration](https://auth0.com/docs/customize/email/smtp-email-providers) in your tenant. A list of fields to include or exclude may also be specified.

        Parameters
        ----------
        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (dependent upon include_fields) from the result. Leave empty to retrieve `name` and `enabled`. Additional fields available include `credentials`, `default_from_address`, and `settings`.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEmailProviderResponseContent
            Email provider successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.emails.provider.get(
            fields="fields",
            include_fields=True,
        )
        """
        _response = self._raw_client.get(fields=fields, include_fields=include_fields, request_options=request_options)
        return _response.data

    def create(
        self,
        *,
        name: EmailProviderNameEnum,
        credentials: EmailProviderCredentialsSchema,
        enabled: typing.Optional[bool] = OMIT,
        default_from_address: typing.Optional[str] = OMIT,
        settings: typing.Optional[EmailSpecificProviderSettingsWithAdditionalProperties] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateEmailProviderResponseContent:
        """
        Create an [email provider](https://auth0.com/docs/email/providers). The `credentials` object
        requires different properties depending on the email provider (which is specified using the `name` property):

        - `mandrill` requires `api_key`
        - `sendgrid` requires `api_key`
        - `sparkpost` requires `api_key`. Optionally, set `region` to `eu` to use
            the SparkPost service hosted in Western Europe; set to `null` to use the SparkPost service hosted in
            North America. `eu` or `null` are the only valid values for `region`.
        - `mailgun` requires `api_key` and `domain`. Optionally, set `region` to
            `eu` to use the Mailgun service hosted in Europe; set to `null` otherwise. `eu` or
            `null` are the only valid values for `region`.
        - `ses` requires `accessKeyId`, `secretAccessKey`, and `region`
        - `smtp` requires `smtp_host`, `smtp_port`, `smtp_user`, and
            `smtp_pass`

        Depending on the type of provider it is possible to specify `settings` object with different configuration
        options, which will be used when sending an email:

        - `smtp` provider, `settings` may contain `headers` object.
            - When using AWS SES SMTP host, you may provide a name of configuration set in
              `X-SES-Configuration-Set` header. Value must be a string.
            - When using Sparkpost host, you may provide value for
              `X-MSYS_API` header. Value must be an object.
        - For `ses` provider, `settings` may contain `message` object, where you can provide
          a name of configuration set in `configuration_set_name` property. Value must be a string.

        Parameters
        ----------
        name : EmailProviderNameEnum

        credentials : EmailProviderCredentialsSchema

        enabled : typing.Optional[bool]
            Whether the provider is enabled (true) or disabled (false).

        default_from_address : typing.Optional[str]
            Email address to use as "from" when no other address specified.

        settings : typing.Optional[EmailSpecificProviderSettingsWithAdditionalProperties]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateEmailProviderResponseContent
            Email provider successfully created.

        Examples
        --------
        from auth0 import Auth0, EmailProviderCredentialsSchemaZero

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.emails.provider.create(
            name="mailgun",
            credentials=EmailProviderCredentialsSchemaZero(
                api_key="api_key",
            ),
        )
        """
        _response = self._raw_client.create(
            name=name,
            credentials=credentials,
            enabled=enabled,
            default_from_address=default_from_address,
            settings=settings,
            request_options=request_options,
        )
        return _response.data

    def delete(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the email provider.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.emails.provider.delete()
        """
        _response = self._raw_client.delete(request_options=request_options)
        return _response.data

    def update(
        self,
        *,
        name: typing.Optional[EmailProviderNameEnum] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        default_from_address: typing.Optional[str] = OMIT,
        credentials: typing.Optional[EmailProviderCredentialsSchema] = OMIT,
        settings: typing.Optional[EmailSpecificProviderSettingsWithAdditionalProperties] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateEmailProviderResponseContent:
        """
        Update an [email provider](https://auth0.com/docs/email/providers). The `credentials` object
        requires different properties depending on the email provider (which is specified using the `name` property):

        - `mandrill` requires `api_key`
        - `sendgrid` requires `api_key`
        - `sparkpost` requires `api_key`. Optionally, set `region` to `eu` to use
            the SparkPost service hosted in Western Europe; set to `null` to use the SparkPost service hosted in
            North America. `eu` or `null` are the only valid values for `region`.
        - `mailgun` requires `api_key` and `domain`. Optionally, set `region` to
            `eu` to use the Mailgun service hosted in Europe; set to `null` otherwise. `eu` or
            `null` are the only valid values for `region`.
        - `ses` requires `accessKeyId`, `secretAccessKey`, and `region`
        - `smtp` requires `smtp_host`, `smtp_port`, `smtp_user`, and
            `smtp_pass`

        Depending on the type of provider it is possible to specify `settings` object with different configuration
        options, which will be used when sending an email:

        - `smtp` provider, `settings` may contain `headers` object.
            - When using AWS SES SMTP host, you may provide a name of configuration set in
              `X-SES-Configuration-Set` header. Value must be a string.
            - When using Sparkpost host, you may provide value for
              `X-MSYS_API` header. Value must be an object.

          For `ses` provider, `settings` may contain `message` object, where you can provide
          a name of configuration set in `configuration_set_name` property. Value must be a string.

        Parameters
        ----------
        name : typing.Optional[EmailProviderNameEnum]

        enabled : typing.Optional[bool]
            Whether the provider is enabled (true) or disabled (false).

        default_from_address : typing.Optional[str]
            Email address to use as "from" when no other address specified.

        credentials : typing.Optional[EmailProviderCredentialsSchema]

        settings : typing.Optional[EmailSpecificProviderSettingsWithAdditionalProperties]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateEmailProviderResponseContent
            Email provider successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.emails.provider.update()
        """
        _response = self._raw_client.update(
            name=name,
            enabled=enabled,
            default_from_address=default_from_address,
            credentials=credentials,
            settings=settings,
            request_options=request_options,
        )
        return _response.data


class AsyncProviderClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawProviderClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawProviderClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawProviderClient
        """
        return self._raw_client

    async def get(
        self,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetEmailProviderResponseContent:
        """
        Retrieve details of the [email provider configuration](https://auth0.com/docs/customize/email/smtp-email-providers) in your tenant. A list of fields to include or exclude may also be specified.

        Parameters
        ----------
        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (dependent upon include_fields) from the result. Leave empty to retrieve `name` and `enabled`. Additional fields available include `credentials`, `default_from_address`, and `settings`.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetEmailProviderResponseContent
            Email provider successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.emails.provider.get(
                fields="fields",
                include_fields=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            fields=fields, include_fields=include_fields, request_options=request_options
        )
        return _response.data

    async def create(
        self,
        *,
        name: EmailProviderNameEnum,
        credentials: EmailProviderCredentialsSchema,
        enabled: typing.Optional[bool] = OMIT,
        default_from_address: typing.Optional[str] = OMIT,
        settings: typing.Optional[EmailSpecificProviderSettingsWithAdditionalProperties] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateEmailProviderResponseContent:
        """
        Create an [email provider](https://auth0.com/docs/email/providers). The `credentials` object
        requires different properties depending on the email provider (which is specified using the `name` property):

        - `mandrill` requires `api_key`
        - `sendgrid` requires `api_key`
        - `sparkpost` requires `api_key`. Optionally, set `region` to `eu` to use
            the SparkPost service hosted in Western Europe; set to `null` to use the SparkPost service hosted in
            North America. `eu` or `null` are the only valid values for `region`.
        - `mailgun` requires `api_key` and `domain`. Optionally, set `region` to
            `eu` to use the Mailgun service hosted in Europe; set to `null` otherwise. `eu` or
            `null` are the only valid values for `region`.
        - `ses` requires `accessKeyId`, `secretAccessKey`, and `region`
        - `smtp` requires `smtp_host`, `smtp_port`, `smtp_user`, and
            `smtp_pass`

        Depending on the type of provider it is possible to specify `settings` object with different configuration
        options, which will be used when sending an email:

        - `smtp` provider, `settings` may contain `headers` object.
            - When using AWS SES SMTP host, you may provide a name of configuration set in
              `X-SES-Configuration-Set` header. Value must be a string.
            - When using Sparkpost host, you may provide value for
              `X-MSYS_API` header. Value must be an object.
        - For `ses` provider, `settings` may contain `message` object, where you can provide
          a name of configuration set in `configuration_set_name` property. Value must be a string.

        Parameters
        ----------
        name : EmailProviderNameEnum

        credentials : EmailProviderCredentialsSchema

        enabled : typing.Optional[bool]
            Whether the provider is enabled (true) or disabled (false).

        default_from_address : typing.Optional[str]
            Email address to use as "from" when no other address specified.

        settings : typing.Optional[EmailSpecificProviderSettingsWithAdditionalProperties]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateEmailProviderResponseContent
            Email provider successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0, EmailProviderCredentialsSchemaZero

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.emails.provider.create(
                name="mailgun",
                credentials=EmailProviderCredentialsSchemaZero(
                    api_key="api_key",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            name=name,
            credentials=credentials,
            enabled=enabled,
            default_from_address=default_from_address,
            settings=settings,
            request_options=request_options,
        )
        return _response.data

    async def delete(self, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete the email provider.

        Parameters
        ----------
        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.emails.provider.delete()


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(request_options=request_options)
        return _response.data

    async def update(
        self,
        *,
        name: typing.Optional[EmailProviderNameEnum] = OMIT,
        enabled: typing.Optional[bool] = OMIT,
        default_from_address: typing.Optional[str] = OMIT,
        credentials: typing.Optional[EmailProviderCredentialsSchema] = OMIT,
        settings: typing.Optional[EmailSpecificProviderSettingsWithAdditionalProperties] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateEmailProviderResponseContent:
        """
        Update an [email provider](https://auth0.com/docs/email/providers). The `credentials` object
        requires different properties depending on the email provider (which is specified using the `name` property):

        - `mandrill` requires `api_key`
        - `sendgrid` requires `api_key`
        - `sparkpost` requires `api_key`. Optionally, set `region` to `eu` to use
            the SparkPost service hosted in Western Europe; set to `null` to use the SparkPost service hosted in
            North America. `eu` or `null` are the only valid values for `region`.
        - `mailgun` requires `api_key` and `domain`. Optionally, set `region` to
            `eu` to use the Mailgun service hosted in Europe; set to `null` otherwise. `eu` or
            `null` are the only valid values for `region`.
        - `ses` requires `accessKeyId`, `secretAccessKey`, and `region`
        - `smtp` requires `smtp_host`, `smtp_port`, `smtp_user`, and
            `smtp_pass`

        Depending on the type of provider it is possible to specify `settings` object with different configuration
        options, which will be used when sending an email:

        - `smtp` provider, `settings` may contain `headers` object.
            - When using AWS SES SMTP host, you may provide a name of configuration set in
              `X-SES-Configuration-Set` header. Value must be a string.
            - When using Sparkpost host, you may provide value for
              `X-MSYS_API` header. Value must be an object.

          For `ses` provider, `settings` may contain `message` object, where you can provide
          a name of configuration set in `configuration_set_name` property. Value must be a string.

        Parameters
        ----------
        name : typing.Optional[EmailProviderNameEnum]

        enabled : typing.Optional[bool]
            Whether the provider is enabled (true) or disabled (false).

        default_from_address : typing.Optional[str]
            Email address to use as "from" when no other address specified.

        credentials : typing.Optional[EmailProviderCredentialsSchema]

        settings : typing.Optional[EmailSpecificProviderSettingsWithAdditionalProperties]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateEmailProviderResponseContent
            Email provider successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.emails.provider.update()


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            name=name,
            enabled=enabled,
            default_from_address=default_from_address,
            credentials=credentials,
            settings=settings,
            request_options=request_options,
        )
        return _response.data



"""Token Verifier module"""
from __future__ import annotations

import json
import time
from typing import TYPE_CHECKING, Any, ClassVar

import jwt
import requests

from .exceptions import TokenValidationError

if TYPE_CHECKING:
    from cryptography.hazmat.primitives.asymmetric.rsa import RSAPublicKey


class SignatureVerifier:
    """Abstract class that will verify a given JSON web token's signature
    using the key fetched internally given its key id.

    Args:
        algorithm (str): The expected signing algorithm (e.g. RS256).
    """

    DISABLE_JWT_CHECKS: ClassVar[dict[str, bool]] = {
        "verify_signature": True,
        "verify_exp": False,
        "verify_nbf": False,
        "verify_iat": False,
        "verify_aud": False,
        "verify_iss": False,
        "require_exp": False,
        "require_iat": False,
        "require_nbf": False,
    }

    def __init__(self, algorithm: str) -> None:
        if not algorithm or type(algorithm) != str:
            raise ValueError("algorithm must be specified.")
        self._algorithm = algorithm

    def _fetch_key(self, key_id: str) -> str | RSAPublicKey:
        """Obtains the key associated to the given key id.
        Must be implemented by subclasses.

        Args:
            key_id (str): The id of the key to fetch.

        Returns:
            the key to use for verifying a cryptographic signature
        """
        raise NotImplementedError

    def _get_kid(self, token: str) -> str | None:
        """Gets the key id from the kid claim of the header of the token

        Args:
            token (str): The JWT to get the header from.

        Raises:
            TokenValidationError: if the token cannot be decoded, the algorithm is invalid
            or the token's signature doesn't match the calculated one.

        Returns:
            the key id or None
        """
        try:
            header = jwt.get_unverified_header(token)
        except jwt.exceptions.DecodeError:
            raise TokenValidationError("token could not be decoded.")

        alg = header.get("alg", None)
        if alg != self._algorithm:
            raise TokenValidationError(
                'Signature algorithm of "{}" is not supported. Expected the token '
                'to be signed with "{}"'.format(alg, self._algorithm)
            )

        return header.get("kid", None)

    def _decode_jwt(self, token: str, secret_or_certificate: str) -> dict[str, Any]:
        """Verifies and decodes the given JSON web token with the given public key or shared secret.

        Args:
            token (str): The JWT to get its signature verified.
            secret_or_certificate (str): The public key or shared secret.

        Raises:
            TokenValidationError: if the token cannot be decoded, the algorithm is invalid
            or the token's signature doesn't match the calculated one.
        """
        try:
            decoded = jwt.decode(
                jwt=token,
                key=secret_or_certificate,
                algorithms=[self._algorithm],
                options=self.DISABLE_JWT_CHECKS,
            )
        except jwt.exceptions.InvalidSignatureError:
            raise TokenValidationError("Invalid token signature.")
        return decoded

    def verify_signature(self, token: str) -> dict[str, Any]:
        """Verifies the signature of the given JSON web token.

        Args:
            token (str): The JWT to get its signature verified.

        Raises:
            TokenValidationError: if the token cannot be decoded, the algorithm is invalid
            or the token's signature doesn't match the calculated one.
        """
        kid = self._get_kid(token)
        if kid is None:
            kid = ""
        secret_or_certificate = self._fetch_key(key_id=kid)

        return self._decode_jwt(token, secret_or_certificate)  # type: ignore[arg-type]


class SymmetricSignatureVerifier(SignatureVerifier):
    """Verifier for HMAC signatures, which rely on shared secrets.

    Args:
        shared_secret (str): The shared secret used to decode the token.
        algorithm (str, optional): The expected signing algorithm. Defaults to "HS256".
    """

    def __init__(self, shared_secret: str, algorithm: str = "HS256") -> None:
        super().__init__(algorithm)
        self._shared_secret = shared_secret

    def _fetch_key(self, key_id: str = "") -> str:
        return self._shared_secret


class JwksFetcher:
    """Class that fetches and holds a JSON web key set.
    This class makes use of an in-memory cache. For it to work properly, define this instance once and re-use it.

    Args:
        jwks_url (str): The url where the JWK set is located.
        cache_ttl (str, optional): The lifetime of the JWK set cache in seconds. Defaults to 600 seconds.
    """

    CACHE_TTL: ClassVar[int] = 600  # 10 min cache lifetime

    def __init__(self, jwks_url: str, cache_ttl: int = CACHE_TTL) -> None:
        self._jwks_url = jwks_url
        self._init_cache(cache_ttl)

    def _init_cache(self, cache_ttl: int) -> None:
        self._cache_value: dict[str, RSAPublicKey] = {}
        self._cache_date = 0.0
        self._cache_ttl = cache_ttl
        self._cache_is_fresh = False

    def _cache_expired(self) -> bool:
        """Checks if the cache is expired

        Returns:
            True if it should use the cache.
        """
        return self._cache_date + self._cache_ttl < time.time()

    def _cache_jwks(self, jwks: dict[str, Any]) -> None:
        """Cache the response of the JWKS request

        Args:
            jwks (dict): The JWKS
        """
        self._cache_value = self._parse_jwks(jwks)
        self._cache_is_fresh = True
        self._cache_date = time.time()

    def _fetch_jwks(self, force: bool = False) -> dict[str, RSAPublicKey]:
        """Attempts to obtain the JWK set from the cache, as long as it's still valid.
        When not, it will perform a network request to the jwks_url to obtain a fresh result
        and update the cache value with it.

        Args:
            force (bool, optional): whether to ignore the cache and force a network request or not. Defaults to False.
        """
        if force or self._cache_expired():
            self._cache_value = {}
            response = requests.get(self._jwks_url)
            if response.ok:
                jwks: dict[str, Any] = response.json()
                self._cache_jwks(jwks)
            return self._cache_value

        self._cache_is_fresh = False
        return self._cache_value

    @staticmethod
    def _parse_jwks(jwks: dict[str, Any]) -> dict[str, RSAPublicKey]:
        """
        Converts a JWK string representation into a binary certificate in PEM format.
        """
        keys: dict[str, RSAPublicKey] = {}

        for key in jwks["keys"]:
            # noinspection PyUnresolvedReferences
            # requirement already includes cryptography -> pyjwt[crypto]
            rsa_key: RSAPublicKey = jwt.algorithms.RSAAlgorithm.from_jwk(
                json.dumps(key)
            )
            keys[key["kid"]] = rsa_key
        return keys

    def get_key(self, key_id: str) -> RSAPublicKey:
        """Obtains the JWK associated with the given key id.

        Args:
            key_id (str): The id of the key to fetch.

        Returns:
            the JWK associated with the given key id.

        Raises:
            TokenValidationError: when a key with that id cannot be found
        """
        keys = self._fetch_jwks()

        if keys and key_id in keys:
            return keys[key_id]

        if not self._cache_is_fresh:
            keys = self._fetch_jwks(force=True)
            if keys and key_id in keys:
                return keys[key_id]
        raise TokenValidationError(f'RSA Public Key with ID "{key_id}" was not found.')


class AsymmetricSignatureVerifier(SignatureVerifier):
    """Verifier for RSA signatures, which rely on public key certificates.

    Args:
        jwks_url (str): The url where the JWK set is located.
        algorithm (str, optional): The expected signing algorithm. Defaults to "RS256".
        cache_ttl (int, optional): The lifetime of the JWK set cache in seconds. Defaults to 600 seconds.
    """

    def __init__(
        self,
        jwks_url: str,
        algorithm: str = "RS256",
        cache_ttl: int = JwksFetcher.CACHE_TTL,
    ) -> None:
        super().__init__(algorithm)
        self._fetcher = JwksFetcher(jwks_url, cache_ttl)

    def _fetch_key(self, key_id: str) -> RSAPublicKey:
        return self._fetcher.get_key(key_id)


class TokenVerifier:
    """Class that verifies ID tokens following the steps defined in the OpenID Connect spec.
    An OpenID Connect ID token is not meant to be consumed until it's verified.

    Args:
        signature_verifier (SignatureVerifier): The instance that knows how to verify the signature.
        issuer (str): The expected issuer claim value.
        audience (str): The expected audience claim value.
        leeway (int, optional): The clock skew to accept when verifying date related claims in seconds.
        Defaults to 60 seconds.
    """

    def __init__(
        self,
        signature_verifier: SignatureVerifier,
        issuer: str,
        audience: str,
        leeway: int = 0,
    ) -> None:
        if not signature_verifier or not isinstance(
            signature_verifier, SignatureVerifier
        ):
            raise TypeError(
                "signature_verifier must be an instance of SignatureVerifier."
            )

        self.iss = issuer
        self.aud = audience
        self.leeway = leeway
        self._sv = signature_verifier
        self._clock = None  # visible for testing

    def verify(
        self,
        token: str,
        nonce: str | None = None,
        max_age: int | None = None,
        organization: str | None = None,
    ) -> dict[str, Any]:
        """Attempts to verify the given ID token, following the steps defined in the OpenID Connect spec.

        Args:
            token (str): The JWT to verify.
            nonce (str, optional): The nonce value sent during authentication.
            max_age (int, optional): The max_age value sent during authentication.
            organization (str, optional): The expected organization ID (org_id) or organization name (org_name) claim value. This should be specified
            when logging in to an organization.

        Returns:
            the decoded payload from the token

        Raises:
            TokenValidationError: when the token cannot be decoded, the token signing algorithm is not the expected one,
            the token signature is invalid or the token has a claim missing or with unexpected value.
        """

        # Verify token presence
        if not token or not isinstance(token, str):
            raise TokenValidationError("ID token is required but missing.")

        # Verify algorithm and signature
        payload = self._sv.verify_signature(token)

        # Verify claims
        self._verify_payload(payload, nonce, max_age, organization)

        return payload

    def _verify_payload(
        self,
        payload: dict[str, Any],
        nonce: str | None = None,
        max_age: int | None = None,
        organization: str | None = None,
    ) -> None:
        # Issuer
        if "iss" not in payload or not isinstance(payload["iss"], str):
            raise TokenValidationError(
                "Issuer (iss) claim must be a string present in the ID token"
            )
        if payload["iss"] != self.iss:
            raise TokenValidationError(
                'Issuer (iss) claim mismatch in the ID token; expected "{}", '
                'found "{}"'.format(self.iss, payload["iss"])
            )

        # Subject
        if "sub" not in payload or not isinstance(payload["sub"], str):
            raise TokenValidationError(
                "Subject (sub) claim must be a string present in the ID token"
            )

        # Audience
        if "aud" not in payload or not isinstance(payload["aud"], (str, list)):
            raise TokenValidationError(
                "Audience (aud) claim must be a string or array of strings present in"
                " the ID token"
            )

        if isinstance(payload["aud"], list) and self.aud not in payload["aud"]:
            payload_audiences = ", ".join(payload["aud"])
            raise TokenValidationError(
                'Audience (aud) claim mismatch in the ID token; expected "{}" but was '
                'not one of "{}"'.format(self.aud, payload_audiences)
            )
        elif isinstance(payload["aud"], str) and payload["aud"] != self.aud:
            raise TokenValidationError(
                'Audience (aud) claim mismatch in the ID token; expected "{}" '
                'but found "{}"'.format(self.aud, payload["aud"])
            )

        # --Time validation (epoch)--
        now = self._clock or time.time()
        leeway = self.leeway

        # Expires at
        if "exp" not in payload or not isinstance(payload["exp"], int):
            raise TokenValidationError(
                "Expiration Time (exp) claim must be a number present in the ID token"
            )

        exp_time = payload["exp"] + leeway
        if now > exp_time:
            raise TokenValidationError(
                "Expiration Time (exp) claim error in the ID token; current time ({})"
                " is after expiration time ({})".format(now, exp_time)
            )

        # Issued at
        if "iat" not in payload or not isinstance(payload["iat"], int):
            raise TokenValidationError(
                "Issued At (iat) claim must be a number present in the ID token"
            )

        # Nonce
        if nonce:
            if "nonce" not in payload or not isinstance(payload["nonce"], str):
                raise TokenValidationError(
                    "Nonce (nonce) claim must be a string present in the ID token"
                )
            if payload["nonce"] != nonce:
                raise TokenValidationError(
                    'Nonce (nonce) claim mismatch in the ID token; expected "{}", '
                    'found "{}"'.format(nonce, payload["nonce"])
                )

        # Organization
        if organization:
            if organization.startswith("org_"):
                if "org_id" not in payload or not isinstance(payload["org_id"], str):
                    raise TokenValidationError(
                        "Organization (org_id) claim must be a string present in the ID"
                        " token"
                    )
                if payload["org_id"] != organization:
                    raise TokenValidationError(
                        "Organization (org_id) claim mismatch in the ID token; expected"
                        ' "{}", found "{}"'.format(organization, payload["org_id"])
                    )
            else:
                if "org_name" not in payload or not isinstance(
                    payload["org_name"], str
                ):
                    raise TokenValidationError(
                        "Organization (org_name) claim must be a string present in the ID"
                        " token"
                    )
                if payload["org_name"] != organization.lower():
                    raise TokenValidationError(
                        "Organization (org_name) claim mismatch in the ID token; expected"
                        ' "{}", found "{}"'.format(organization, payload["org_name"])
                    )

        # Authorized party
        if isinstance(payload["aud"], list) and len(payload["aud"]) > 1:
            if "azp" not in payload or not isinstance(payload["azp"], str):
                raise TokenValidationError(
                    "Authorized Party (azp) claim must be a string present in the ID"
                    " token when Audience (aud) claim has multiple values"
                )
            if payload["azp"] != self.aud:
                raise TokenValidationError(
                    "Authorized Party (azp) claim mismatch in the ID token; expected"
                    ' "{}", found "{}"'.format(self.aud, payload["azp"])
                )

        # Authentication time
        if max_age:
            if "auth_time" not in payload or not isinstance(payload["auth_time"], int):
                raise TokenValidationError(
                    "Authentication Time (auth_time) claim must be a number present in"
                    " the ID token when Max Age (max_age) is specified"
                )

            auth_valid_until = payload["auth_time"] + max_age + leeway
            if now > auth_valid_until:
                raise TokenValidationError(
                    "Authentication Time (auth_time) claim in the ID token indicates"
                    " that too much time has passed since the last end-user"
                    " authentication. Current time ({}) is after last auth at ({})".format(
                        now, auth_valid_until
                    )
                )



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionCustomHeadersOAuth2 = typing.Dict[str, str]
"""
Custom HTTP headers sent with token exchange requests to the identity provider's token endpoint. Provided as key-value pairs (e.g., {'X-Custom-Header': 'value'}). Auth0's User-Agent header is always included by default.
"""



# This file was auto-generated by Fern from our API Definition.

ConnectionSigningCertificateDerSaml = str
"""
X.509 signing certificate from the identity provider in .der format. Used to validate signatures in SAML Responses and Assertions. This is an alternative to signingCert and is kept for backward compatibility. Prefer using signingCert instead.
"""



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionAuthParamsMap = typing.Dict[str, str]
"""
Maps parameter names from Auth0's /authorize endpoint to the identity provider's authorization endpoint parameters. For example, mapping 'audience' to 'resource' transforms the parameter name during authorization requests. Applied after authParams merging. See https://auth0.com/docs/authenticate/identity-providers/social-identity-providers/oauth2#pass-dynamic-parameters
"""



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .network_acl_rule import NetworkAclRule


class NetworkAclsResponseContent(UniversalBaseModel):
    id: typing.Optional[str] = None
    description: typing.Optional[str] = None
    active: typing.Optional[bool] = None
    priority: typing.Optional[float] = None
    rule: typing.Optional[NetworkAclRule] = None
    created_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the Network ACL Configuration was created
    """

    updated_at: typing.Optional[str] = pydantic.Field(default=None)
    """
    The timestamp when the Network ACL Configuration was last updated
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_mailjet_send_email_params_content import FlowActionMailjetSendEmailParamsContent
from .flow_action_mailjet_send_email_params_template_id import FlowActionMailjetSendEmailParamsTemplateId

FlowActionMailjetSendEmailParams = typing.Union[
    FlowActionMailjetSendEmailParamsContent, FlowActionMailjetSendEmailParamsTemplateId
]



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionScopesSupported = typing.Optional[typing.List[str]]
"""
A list of the OAuth 2.0 [RFC6749] scope values that this server supports. The server MUST support the openid scope value. Servers MAY choose not to advertise some supported scope values even when this parameter is used, although those defined in [OpenID.Core] SHOULD be listed, if supported. RECOMMENDED but not REQUIRED
"""



# This file was auto-generated by Fern from our API Definition.

import typing

DeviceCredentialTypeEnum = typing.Union[
    typing.Literal["public_key", "refresh_token", "rotating_refresh_token"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .user_block_identifier import UserBlockIdentifier


class ListUserBlocksResponseContent(UniversalBaseModel):
    blocked_for: typing.Optional[typing.List[UserBlockIdentifier]] = pydantic.Field(default=None)
    """
    Array of identifier + IP address pairs.  IP address is optional, and may be omitted in certain circumstances (such as Account Lockout mode).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



from .conftest import get_client, verify_request_count


def test_resourceServers_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "resource_servers.list_.0"
    client = get_client(test_id)
    client.resource_servers.list(
        identifiers=["identifiers"],
        page=1,
        per_page=1,
        include_totals=True,
        include_fields=True,
    )
    verify_request_count(
        test_id,
        "GET",
        "/resource-servers",
        {
            "identifiers": "identifiers",
            "page": "1",
            "per_page": "1",
            "include_totals": "true",
            "include_fields": "true",
        },
        1,
    )


def test_resourceServers_create() -> None:
    """Test create endpoint with WireMock"""
    test_id = "resource_servers.create.0"
    client = get_client(test_id)
    client.resource_servers.create(
        identifier="identifier",
    )
    verify_request_count(test_id, "POST", "/resource-servers", None, 1)


def test_resourceServers_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "resource_servers.get.0"
    client = get_client(test_id)
    client.resource_servers.get(
        id="id",
        include_fields=True,
    )
    verify_request_count(test_id, "GET", "/resource-servers/id", {"include_fields": "true"}, 1)


def test_resourceServers_delete() -> None:
    """Test delete endpoint with WireMock"""
    test_id = "resource_servers.delete.0"
    client = get_client(test_id)
    client.resource_servers.delete(
        id="id",
    )
    verify_request_count(test_id, "DELETE", "/resource-servers/id", None, 1)


def test_resourceServers_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "resource_servers.update.0"
    client = get_client(test_id)
    client.resource_servers.update(
        id="id",
    )
    verify_request_count(test_id, "PATCH", "/resource-servers/id", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .event_stream_cloud_event_connection_updated_object_1_options_attribute_map_attributes import (
    EventStreamCloudEventConnectionUpdatedObject1OptionsAttributeMapAttributes,
)
from .event_stream_cloud_event_connection_updated_object_1_options_attribute_map_mapping_mode_enum import (
    EventStreamCloudEventConnectionUpdatedObject1OptionsAttributeMapMappingModeEnum,
)


class EventStreamCloudEventConnectionUpdatedObject1OptionsAttributeMap(UniversalBaseModel):
    """
    Mapping of claims received from the identity provider (IdP)
    """

    attributes: typing.Optional[EventStreamCloudEventConnectionUpdatedObject1OptionsAttributeMapAttributes] = None
    userinfo_scope: typing.Optional[str] = pydantic.Field(default=None)
    """
    Scopes to send to the IdP's Userinfo endpoint
    """

    mapping_mode: typing.Optional[EventStreamCloudEventConnectionUpdatedObject1OptionsAttributeMapMappingModeEnum] = (
        None
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



import unittest
from unittest import mock

from auth0.authentication.users import Users


class TestUsers(unittest.TestCase):
    @mock.patch("auth0.authentication.rest.RestClient.get")
    def test_userinfo(self, mock_get):
        u = Users("my.domain.com")

        u.userinfo(access_token="atk")

        mock_get.assert_called_with(
            url="https://my.domain.com/userinfo",
            headers={"Authorization": "Bearer atk"},
        )



from .conftest import get_client, verify_request_count


def test_tickets_verify_email() -> None:
    """Test verifyEmail endpoint with WireMock"""
    test_id = "tickets.verify_email.0"
    client = get_client(test_id)
    client.tickets.verify_email(
        user_id="user_id",
    )
    verify_request_count(test_id, "POST", "/tickets/email-verification", None, 1)


def test_tickets_change_password() -> None:
    """Test changePassword endpoint with WireMock"""
    test_id = "tickets.change_password.0"
    client = get_client(test_id)
    client.tickets.change_password()
    verify_request_count(test_id, "POST", "/tickets/password-change", None, 1)



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionNonPersistentAttrs = typing.List[str]
"""
An array of user fields that should not be stored in the Auth0 database (https://auth0.com/docs/security/data-security/denylist)
"""



# This file was auto-generated by Fern from our API Definition.

import typing

from .event_stream_cloud_event_group_updated_object_0 import EventStreamCloudEventGroupUpdatedObject0
from .event_stream_cloud_event_group_updated_object_1 import EventStreamCloudEventGroupUpdatedObject1
from .event_stream_cloud_event_group_updated_object_2 import EventStreamCloudEventGroupUpdatedObject2

EventStreamCloudEventGroupUpdatedObject = typing.Union[
    EventStreamCloudEventGroupUpdatedObject0,
    EventStreamCloudEventGroupUpdatedObject1,
    EventStreamCloudEventGroupUpdatedObject2,
]



from .conftest import get_client, verify_request_count


def test_users_multifactor_invalidate_remember_browser() -> None:
    """Test invalidateRememberBrowser endpoint with WireMock"""
    test_id = "users.multifactor.invalidate_remember_browser.0"
    client = get_client(test_id)
    client.users.multifactor.invalidate_remember_browser(
        id="id",
    )
    verify_request_count(test_id, "POST", "/users/id/multifactor/actions/invalidate-remember-browser", None, 1)


def test_users_multifactor_delete_provider() -> None:
    """Test deleteProvider endpoint with WireMock"""
    test_id = "users.multifactor.delete_provider.0"
    client = get_client(test_id)
    client.users.multifactor.delete_provider(
        id="id",
        provider="duo",
    )
    verify_request_count(test_id, "DELETE", "/users/id/multifactor/duo", None, 1)



# This file was auto-generated by Fern from our API Definition.

ConnectionIsDomainConnection = bool
"""
<code>true</code> promotes to a domain-level connection so that third-party applications can use it. <code>false</code> does not promote the connection, so only first-party applications with the connection enabled can use it. (Defaults to <code>false</code>.)
"""



# This file was auto-generated by Fern from our API Definition.

from .connection_ping_federate_base_url_ping_federate import ConnectionPingFederateBaseUrlPingFederate

ConnectionPingFederateBaseUrl = ConnectionPingFederateBaseUrlPingFederate
"""
URL provided by PingFederate which returns information used for creating the connection
"""



from .conftest import get_client, verify_request_count


def test_users_effectivePermissions_sources_roles_list_() -> None:
    """Test list endpoint with WireMock"""
    test_id = "users.effective_permissions.sources.roles.list_.0"
    client = get_client(test_id)
    client.users.effective_permissions.sources.roles.list(
        id="id",
        from_="from",
        take=1,
        resource_server_identifier="resource_server_identifier",
        permission_name="permission_name",
    )
    verify_request_count(
        test_id,
        "GET",
        "/users/id/effective-permissions/sources/effective-roles",
        {
            "from": "from",
            "take": "1",
            "resource_server_identifier": "resource_server_identifier",
            "permission_name": "permission_name",
        },
        1,
    )



# This file was auto-generated by Fern from our API Definition.

import typing

from .patch_rate_limit_policy_configuration_request_content_action import (
    PatchRateLimitPolicyConfigurationRequestContentAction,
)
from .patch_rate_limit_policy_configuration_request_content_one import (
    PatchRateLimitPolicyConfigurationRequestContentOne,
)
from .patch_rate_limit_policy_configuration_request_content_zero import (
    PatchRateLimitPolicyConfigurationRequestContentZero,
)

PatchRateLimitPolicyConfigurationRequestContent = typing.Union[
    PatchRateLimitPolicyConfigurationRequestContentZero,
    PatchRateLimitPolicyConfigurationRequestContentOne,
    PatchRateLimitPolicyConfigurationRequestContentAction,
]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.pagination import AsyncPager, SyncPager
from ..core.request_options import RequestOptions
from ..types.app_metadata import AppMetadata
from ..types.create_user_response_content import CreateUserResponseContent
from ..types.get_user_response_content import GetUserResponseContent
from ..types.list_users_offset_paginated_response_content import ListUsersOffsetPaginatedResponseContent
from ..types.regenerate_users_recovery_code_response_content import RegenerateUsersRecoveryCodeResponseContent
from ..types.search_engine_versions_enum import SearchEngineVersionsEnum
from ..types.update_user_response_content import UpdateUserResponseContent
from ..types.user_metadata import UserMetadata
from ..types.user_response_schema import UserResponseSchema
from .raw_client import AsyncRawUsersClient, RawUsersClient

if typing.TYPE_CHECKING:
    from .authentication_methods.client import AsyncAuthenticationMethodsClient, AuthenticationMethodsClient
    from .authenticators.client import AsyncAuthenticatorsClient, AuthenticatorsClient
    from .connected_accounts.client import AsyncConnectedAccountsClient, ConnectedAccountsClient
    from .effective_permissions.client import AsyncEffectivePermissionsClient, EffectivePermissionsClient
    from .effective_roles.client import AsyncEffectiveRolesClient, EffectiveRolesClient
    from .enrollments.client import AsyncEnrollmentsClient, EnrollmentsClient
    from .groups.client import AsyncGroupsClient, GroupsClient
    from .identities.client import AsyncIdentitiesClient, IdentitiesClient
    from .logs.client import AsyncLogsClient, LogsClient
    from .multifactor.client import AsyncMultifactorClient, MultifactorClient
    from .organizations.client import AsyncOrganizationsClient, OrganizationsClient
    from .permissions.client import AsyncPermissionsClient, PermissionsClient
    from .refresh_token.client import AsyncRefreshTokenClient, RefreshTokenClient
    from .risk_assessments.client import AsyncRiskAssessmentsClient, RiskAssessmentsClient
    from .roles.client import AsyncRolesClient, RolesClient
    from .sessions.client import AsyncSessionsClient, SessionsClient
# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UsersClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUsersClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._authentication_methods: typing.Optional[AuthenticationMethodsClient] = None
        self._authenticators: typing.Optional[AuthenticatorsClient] = None
        self._connected_accounts: typing.Optional[ConnectedAccountsClient] = None
        self._effective_permissions: typing.Optional[EffectivePermissionsClient] = None
        self._effective_roles: typing.Optional[EffectiveRolesClient] = None
        self._enrollments: typing.Optional[EnrollmentsClient] = None
        self._groups: typing.Optional[GroupsClient] = None
        self._identities: typing.Optional[IdentitiesClient] = None
        self._logs: typing.Optional[LogsClient] = None
        self._multifactor: typing.Optional[MultifactorClient] = None
        self._organizations: typing.Optional[OrganizationsClient] = None
        self._permissions: typing.Optional[PermissionsClient] = None
        self._risk_assessments: typing.Optional[RiskAssessmentsClient] = None
        self._roles: typing.Optional[RolesClient] = None
        self._refresh_token: typing.Optional[RefreshTokenClient] = None
        self._sessions: typing.Optional[SessionsClient] = None

    @property
    def with_raw_response(self) -> RawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUsersClient
        """
        return self._raw_client

    def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        sort: typing.Optional[str] = None,
        connection: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        q: typing.Optional[str] = None,
        search_engine: typing.Optional[SearchEngineVersionsEnum] = None,
        primary_order: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[UserResponseSchema, ListUsersOffsetPaginatedResponseContent]:
        """
        Retrieve details of users. It is possible to:

        - Specify a search criteria for users
        - Sort the users to be returned
        - Select the fields to be returned
        - Specify the number of users to retrieve per page and the page index



        The `q` query parameter can be used to get users that match the specified criteria [using query string syntax.](https://auth0.com/docs/users/search/v3/query-syntax)

        [Learn more about searching for users.](https://auth0.com/docs/users/search/v3)

        Read about [best practices](https://auth0.com/docs/users/search/best-practices) when working with the API endpoints for retrieving users.



        Auth0 limits the number of users you can return. If you exceed this threshold, please redefine your search, use the [export job](https://auth0.com/docs/api/management/v2#!/Jobs/post_users_exports), or the [User Import / Export](https://auth0.com/docs/extensions/user-import-export) extension.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        sort : typing.Optional[str]
            Field to sort by. Use <code>field:order</code> where order is <code>1</code> for ascending and <code>-1</code> for descending. e.g. <code>created_at:1</code>

        connection : typing.Optional[str]
            Connection filter. Only applies when using <code>search_engine=v1</code>. To filter by connection with <code>search_engine=v2|v3</code>, use <code>q=identities.connection:"connection_name"</code>

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        q : typing.Optional[str]
            Query in <a target='_new' href ='https://lucene.apache.org/core/2_9_4/queryparsersyntax.html'>Lucene query string syntax</a>. Some query types cannot be used on metadata fields, for details see <a href='https://auth0.com/docs/users/search/v3/query-syntax#searchable-fields'>Searchable Fields</a>.

        search_engine : typing.Optional[SearchEngineVersionsEnum]
            The version of the search engine

        primary_order : typing.Optional[bool]
            If true (default), results are returned in a deterministic order. If false, results may be returned in a non-deterministic order, which can enhance performance for complex queries targeting a small number of users. Set to false only when consistent ordering and pagination is not required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[UserResponseSchema, ListUsersOffsetPaginatedResponseContent]
            Users successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        response = client.users.list(
            page=1,
            per_page=1,
            include_totals=True,
            sort="sort",
            connection="connection",
            fields="fields",
            include_fields=True,
            q="q",
            search_engine="v1",
            primary_order=True,
        )
        for item in response:
            yield item
        # alternatively, you can paginate page-by-page
        for page in response.iter_pages():
            yield page
        """
        return self._raw_client.list(
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            sort=sort,
            connection=connection,
            fields=fields,
            include_fields=include_fields,
            q=q,
            search_engine=search_engine,
            primary_order=primary_order,
            request_options=request_options,
        )

    def create(
        self,
        *,
        connection: str,
        email: typing.Optional[str] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        user_metadata: typing.Optional[UserMetadata] = OMIT,
        blocked: typing.Optional[bool] = OMIT,
        email_verified: typing.Optional[bool] = OMIT,
        phone_verified: typing.Optional[bool] = OMIT,
        app_metadata: typing.Optional[AppMetadata] = OMIT,
        given_name: typing.Optional[str] = OMIT,
        family_name: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        nickname: typing.Optional[str] = OMIT,
        picture: typing.Optional[str] = OMIT,
        user_id: typing.Optional[str] = OMIT,
        password: typing.Optional[str] = OMIT,
        verify_email: typing.Optional[bool] = OMIT,
        username: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateUserResponseContent:
        """
        Create a new user for a given [database](https://auth0.com/docs/connections/database) or [passwordless](https://auth0.com/docs/connections/passwordless) connection.

        Note: `connection` is required but other parameters such as `email` and `password` are dependent upon the type of connection.

        Parameters
        ----------
        connection : str
            Name of the connection this user should be created in.

        email : typing.Optional[str]
            The user's email.

        phone_number : typing.Optional[str]
            The user's phone number (following the E.164 recommendation).

        user_metadata : typing.Optional[UserMetadata]

        blocked : typing.Optional[bool]
            Whether this user was blocked by an administrator (true) or not (false).

        email_verified : typing.Optional[bool]
            Whether this email address is verified (true) or unverified (false). User will receive a verification email after creation if `email_verified` is false or not specified

        phone_verified : typing.Optional[bool]
            Whether this phone number has been verified (true) or not (false).

        app_metadata : typing.Optional[AppMetadata]

        given_name : typing.Optional[str]
            The user's given name(s).

        family_name : typing.Optional[str]
            The user's family name(s).

        name : typing.Optional[str]
            The user's full name.

        nickname : typing.Optional[str]
            The user's nickname.

        picture : typing.Optional[str]
            A URI pointing to the user's picture.

        user_id : typing.Optional[str]
            The external user's id provided by the identity provider.

        password : typing.Optional[str]
            Initial password for this user. Only valid for auth0 connection strategy.

        verify_email : typing.Optional[bool]
            Whether the user will receive a verification email after creation (true) or no email (false). Overrides behavior of `email_verified` parameter.

        username : typing.Optional[str]
            The user's username. Only valid if the connection requires a username.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateUserResponseContent
            User successfully created.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.create(
            connection="connection",
        )
        """
        _response = self._raw_client.create(
            connection=connection,
            email=email,
            phone_number=phone_number,
            user_metadata=user_metadata,
            blocked=blocked,
            email_verified=email_verified,
            phone_verified=phone_verified,
            app_metadata=app_metadata,
            given_name=given_name,
            family_name=family_name,
            name=name,
            nickname=nickname,
            picture=picture,
            user_id=user_id,
            password=password,
            verify_email=verify_email,
            username=username,
            request_options=request_options,
        )
        return _response.data

    def list_users_by_email(
        self,
        *,
        email: str,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[UserResponseSchema]:
        """
        Find users by email. If Auth0 is the identity provider (idP), the email address associated with a user is saved in lower case, regardless of how you initially provided it.

        For example, if you register a user as JohnSmith@example.com, Auth0 saves the user's email as johnsmith@example.com.

        Therefore, when using this endpoint, make sure that you are searching for users via email addresses using the correct case.

        Parameters
        ----------
        email : str
            Email address to search for (case-sensitive).

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false). Defaults to true.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[UserResponseSchema]
            Users successfully searched.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.list_users_by_email(
            fields="fields",
            include_fields=True,
            email="email",
        )
        """
        _response = self._raw_client.list_users_by_email(
            email=email, fields=fields, include_fields=include_fields, request_options=request_options
        )
        return _response.data

    def get(
        self,
        id: str,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetUserResponseContent:
        """
        Retrieve user details. A list of fields to include or exclude may also be specified. For more information, see [Retrieve Users with the Get Users Endpoint](https://auth0.com/docs/manage-users/user-search/retrieve-users-with-get-users-endpoint).

        Parameters
        ----------
        id : str
            ID of the user to retrieve.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetUserResponseContent
            User successfully retrieved.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.get(
            id="id",
            fields="fields",
            include_fields=True,
        )
        """
        _response = self._raw_client.get(
            id, fields=fields, include_fields=include_fields, request_options=request_options
        )
        return _response.data

    def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a user by user ID. This action cannot be undone. For Auth0 Dashboard instructions, see [Delete Users](https://auth0.com/docs/manage-users/user-accounts/delete-users).

        Parameters
        ----------
        id : str
            ID of the user to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.delete(
            id="id",
        )
        """
        _response = self._raw_client.delete(id, request_options=request_options)
        return _response.data

    def update(
        self,
        id: str,
        *,
        blocked: typing.Optional[bool] = OMIT,
        email_verified: typing.Optional[bool] = OMIT,
        email: typing.Optional[str] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        phone_verified: typing.Optional[bool] = OMIT,
        user_metadata: typing.Optional[UserMetadata] = OMIT,
        app_metadata: typing.Optional[AppMetadata] = OMIT,
        given_name: typing.Optional[str] = OMIT,
        family_name: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        nickname: typing.Optional[str] = OMIT,
        picture: typing.Optional[str] = OMIT,
        verify_email: typing.Optional[bool] = OMIT,
        verify_phone_number: typing.Optional[bool] = OMIT,
        password: typing.Optional[str] = OMIT,
        connection: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        username: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateUserResponseContent:
        """
        Update a user.

        These are the attributes that can be updated at the root level:

        - app_metadata
        - blocked
        - email
        - email_verified
        - family_name
        - given_name
        - name
        - nickname
        - password
        - phone_number
        - phone_verified
        - picture
        - username
        - user_metadata
        - verify_email

        Some considerations:

        - The properties of the new object will replace the old ones.
        - The metadata fields are an exception to this rule (`user_metadata` and `app_metadata`). These properties are merged instead of being replaced but be careful, the merge only occurs on the first level.
        - If you are updating `email`, `email_verified`, `phone_number`, `phone_verified`, `username` or `password` of a secondary identity, you need to specify the `connection` property too.
        - If you are updating `email` or `phone_number` you can specify, optionally, the `client_id` property.
        - Updating `email_verified` is not supported for enterprise and passwordless sms connections.
        - Updating the `blocked` to `false` does not affect the user's blocked state from an excessive amount of incorrectly provided credentials. Use the "Unblock a user" endpoint from the "User Blocks" API to change the user's state.
        - Supported attributes can be unset by supplying `null` as the value.

        **Updating a field (non-metadata property)**

        To mark the email address of a user as verified, the body to send should be:

        ```json
        { "email_verified": true }
        ```

        **Updating a user metadata root property**

        Let's assume that our test user has the following `user_metadata`:

        ```json
        { "user_metadata" : { "profileCode": 1479 } }
        ```

        To add the field `addresses` the body to send should be:

        ```json
        { "user_metadata" : { "addresses": {"work_address": "100 Industrial Way"} }}
        ```

        The modified object ends up with the following `user_metadata` property:

        ```json
        {
          "user_metadata": {
            "profileCode": 1479,
            "addresses": { "work_address": "100 Industrial Way" }
          }
        }
        ```

        **Updating an inner user metadata property**

        If there's existing user metadata to which we want to add  `"home_address": "742 Evergreen Terrace"` (using the `addresses` property) we should send the whole `addresses` object. Since this is a first-level object, the object will be merged in, but its own properties will not be. The body to send should be:

        ```json
        {
          "user_metadata": {
            "addresses": {
              "work_address": "100 Industrial Way",
              "home_address": "742 Evergreen Terrace"
            }
          }
        }
        ```

        The modified object ends up with the following `user_metadata` property:

        ```json
        {
          "user_metadata": {
            "profileCode": 1479,
            "addresses": {
              "work_address": "100 Industrial Way",
              "home_address": "742 Evergreen Terrace"
            }
          }
        }
        ```

        Parameters
        ----------
        id : str
            ID of the user to update.

        blocked : typing.Optional[bool]
            Whether this user was blocked by an administrator (true) or not (false).

        email_verified : typing.Optional[bool]
            Whether this email address is verified (true) or unverified (false). If set to false the user will not receive a verification email unless `verify_email` is set to true.

        email : typing.Optional[str]
            Email address of this user.

        phone_number : typing.Optional[str]
            The user's phone number (following the E.164 recommendation).

        phone_verified : typing.Optional[bool]
            Whether this phone number has been verified (true) or not (false).

        user_metadata : typing.Optional[UserMetadata]
            User metadata to which this user has read/write access.

        app_metadata : typing.Optional[AppMetadata]
            User metadata to which this user has read-only access.

        given_name : typing.Optional[str]
            Given name/first name/forename of this user.

        family_name : typing.Optional[str]
            Family name/last name/surname of this user.

        name : typing.Optional[str]
            Name of this user.

        nickname : typing.Optional[str]
            Preferred nickname or alias of this user.

        picture : typing.Optional[str]
            URL to picture, photo, or avatar of this user.

        verify_email : typing.Optional[bool]
            Whether this user will receive a verification email after creation (true) or no email (false). Overrides behavior of `email_verified` parameter.

        verify_phone_number : typing.Optional[bool]
            Whether this user will receive a text after changing the phone number (true) or no text (false). Only valid when changing phone number for SMS connections.

        password : typing.Optional[str]
            New password for this user. Only valid for database connections.

        connection : typing.Optional[str]
            Name of the connection to target for this user update.

        client_id : typing.Optional[str]
            Auth0 client ID. Only valid when updating email address.

        username : typing.Optional[str]
            The user's username. Only valid if the connection requires a username.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateUserResponseContent
            User successfully updated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.update(
            id="id",
        )
        """
        _response = self._raw_client.update(
            id,
            blocked=blocked,
            email_verified=email_verified,
            email=email,
            phone_number=phone_number,
            phone_verified=phone_verified,
            user_metadata=user_metadata,
            app_metadata=app_metadata,
            given_name=given_name,
            family_name=family_name,
            name=name,
            nickname=nickname,
            picture=picture,
            verify_email=verify_email,
            verify_phone_number=verify_phone_number,
            password=password,
            connection=connection,
            client_id=client_id,
            username=username,
            request_options=request_options,
        )
        return _response.data

    def regenerate_recovery_code(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RegenerateUsersRecoveryCodeResponseContent:
        """
        Remove an existing multi-factor authentication (MFA) [recovery code](https://auth0.com/docs/secure/multi-factor-authentication/reset-user-mfa) and generate a new one. If a user cannot access the original device or account used for MFA enrollment, they can use a recovery code to authenticate.

        Parameters
        ----------
        id : str
            ID of the user to regenerate a multi-factor authentication recovery code for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RegenerateUsersRecoveryCodeResponseContent
            New recovery code successfully generated.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.regenerate_recovery_code(
            id="id",
        )
        """
        _response = self._raw_client.regenerate_recovery_code(id, request_options=request_options)
        return _response.data

    def revoke_access(
        self,
        id: str,
        *,
        session_id: typing.Optional[str] = OMIT,
        preserve_refresh_tokens: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Revokes selected resources related to a user (sessions, refresh tokens, ...).

        Parameters
        ----------
        id : str
            ID of the user.

        session_id : typing.Optional[str]
            ID of the session to revoke.

        preserve_refresh_tokens : typing.Optional[bool]
            Whether to preserve the refresh tokens associated with the session.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.users.revoke_access(
            id="id",
        )
        """
        _response = self._raw_client.revoke_access(
            id, session_id=session_id, preserve_refresh_tokens=preserve_refresh_tokens, request_options=request_options
        )
        return _response.data

    @property
    def authentication_methods(self):
        if self._authentication_methods is None:
            from .authentication_methods.client import AuthenticationMethodsClient  # noqa: E402

            self._authentication_methods = AuthenticationMethodsClient(client_wrapper=self._client_wrapper)
        return self._authentication_methods

    @property
    def authenticators(self):
        if self._authenticators is None:
            from .authenticators.client import AuthenticatorsClient  # noqa: E402

            self._authenticators = AuthenticatorsClient(client_wrapper=self._client_wrapper)
        return self._authenticators

    @property
    def connected_accounts(self):
        if self._connected_accounts is None:
            from .connected_accounts.client import ConnectedAccountsClient  # noqa: E402

            self._connected_accounts = ConnectedAccountsClient(client_wrapper=self._client_wrapper)
        return self._connected_accounts

    @property
    def effective_permissions(self):
        if self._effective_permissions is None:
            from .effective_permissions.client import EffectivePermissionsClient  # noqa: E402

            self._effective_permissions = EffectivePermissionsClient(client_wrapper=self._client_wrapper)
        return self._effective_permissions

    @property
    def effective_roles(self):
        if self._effective_roles is None:
            from .effective_roles.client import EffectiveRolesClient  # noqa: E402

            self._effective_roles = EffectiveRolesClient(client_wrapper=self._client_wrapper)
        return self._effective_roles

    @property
    def enrollments(self):
        if self._enrollments is None:
            from .enrollments.client import EnrollmentsClient  # noqa: E402

            self._enrollments = EnrollmentsClient(client_wrapper=self._client_wrapper)
        return self._enrollments

    @property
    def groups(self):
        if self._groups is None:
            from .groups.client import GroupsClient  # noqa: E402

            self._groups = GroupsClient(client_wrapper=self._client_wrapper)
        return self._groups

    @property
    def identities(self):
        if self._identities is None:
            from .identities.client import IdentitiesClient  # noqa: E402

            self._identities = IdentitiesClient(client_wrapper=self._client_wrapper)
        return self._identities

    @property
    def logs(self):
        if self._logs is None:
            from .logs.client import LogsClient  # noqa: E402

            self._logs = LogsClient(client_wrapper=self._client_wrapper)
        return self._logs

    @property
    def multifactor(self):
        if self._multifactor is None:
            from .multifactor.client import MultifactorClient  # noqa: E402

            self._multifactor = MultifactorClient(client_wrapper=self._client_wrapper)
        return self._multifactor

    @property
    def organizations(self):
        if self._organizations is None:
            from .organizations.client import OrganizationsClient  # noqa: E402

            self._organizations = OrganizationsClient(client_wrapper=self._client_wrapper)
        return self._organizations

    @property
    def permissions(self):
        if self._permissions is None:
            from .permissions.client import PermissionsClient  # noqa: E402

            self._permissions = PermissionsClient(client_wrapper=self._client_wrapper)
        return self._permissions

    @property
    def risk_assessments(self):
        if self._risk_assessments is None:
            from .risk_assessments.client import RiskAssessmentsClient  # noqa: E402

            self._risk_assessments = RiskAssessmentsClient(client_wrapper=self._client_wrapper)
        return self._risk_assessments

    @property
    def roles(self):
        if self._roles is None:
            from .roles.client import RolesClient  # noqa: E402

            self._roles = RolesClient(client_wrapper=self._client_wrapper)
        return self._roles

    @property
    def refresh_token(self):
        if self._refresh_token is None:
            from .refresh_token.client import RefreshTokenClient  # noqa: E402

            self._refresh_token = RefreshTokenClient(client_wrapper=self._client_wrapper)
        return self._refresh_token

    @property
    def sessions(self):
        if self._sessions is None:
            from .sessions.client import SessionsClient  # noqa: E402

            self._sessions = SessionsClient(client_wrapper=self._client_wrapper)
        return self._sessions


class AsyncUsersClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUsersClient(client_wrapper=client_wrapper)
        self._client_wrapper = client_wrapper
        self._authentication_methods: typing.Optional[AsyncAuthenticationMethodsClient] = None
        self._authenticators: typing.Optional[AsyncAuthenticatorsClient] = None
        self._connected_accounts: typing.Optional[AsyncConnectedAccountsClient] = None
        self._effective_permissions: typing.Optional[AsyncEffectivePermissionsClient] = None
        self._effective_roles: typing.Optional[AsyncEffectiveRolesClient] = None
        self._enrollments: typing.Optional[AsyncEnrollmentsClient] = None
        self._groups: typing.Optional[AsyncGroupsClient] = None
        self._identities: typing.Optional[AsyncIdentitiesClient] = None
        self._logs: typing.Optional[AsyncLogsClient] = None
        self._multifactor: typing.Optional[AsyncMultifactorClient] = None
        self._organizations: typing.Optional[AsyncOrganizationsClient] = None
        self._permissions: typing.Optional[AsyncPermissionsClient] = None
        self._risk_assessments: typing.Optional[AsyncRiskAssessmentsClient] = None
        self._roles: typing.Optional[AsyncRolesClient] = None
        self._refresh_token: typing.Optional[AsyncRefreshTokenClient] = None
        self._sessions: typing.Optional[AsyncSessionsClient] = None

    @property
    def with_raw_response(self) -> AsyncRawUsersClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUsersClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        include_totals: typing.Optional[bool] = True,
        sort: typing.Optional[str] = None,
        connection: typing.Optional[str] = None,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        q: typing.Optional[str] = None,
        search_engine: typing.Optional[SearchEngineVersionsEnum] = None,
        primary_order: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[UserResponseSchema, ListUsersOffsetPaginatedResponseContent]:
        """
        Retrieve details of users. It is possible to:

        - Specify a search criteria for users
        - Sort the users to be returned
        - Select the fields to be returned
        - Specify the number of users to retrieve per page and the page index



        The `q` query parameter can be used to get users that match the specified criteria [using query string syntax.](https://auth0.com/docs/users/search/v3/query-syntax)

        [Learn more about searching for users.](https://auth0.com/docs/users/search/v3)

        Read about [best practices](https://auth0.com/docs/users/search/best-practices) when working with the API endpoints for retrieving users.



        Auth0 limits the number of users you can return. If you exceed this threshold, please redefine your search, use the [export job](https://auth0.com/docs/api/management/v2#!/Jobs/post_users_exports), or the [User Import / Export](https://auth0.com/docs/extensions/user-import-export) extension.

        Parameters
        ----------
        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        sort : typing.Optional[str]
            Field to sort by. Use <code>field:order</code> where order is <code>1</code> for ascending and <code>-1</code> for descending. e.g. <code>created_at:1</code>

        connection : typing.Optional[str]
            Connection filter. Only applies when using <code>search_engine=v1</code>. To filter by connection with <code>search_engine=v2|v3</code>, use <code>q=identities.connection:"connection_name"</code>

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        q : typing.Optional[str]
            Query in <a target='_new' href ='https://lucene.apache.org/core/2_9_4/queryparsersyntax.html'>Lucene query string syntax</a>. Some query types cannot be used on metadata fields, for details see <a href='https://auth0.com/docs/users/search/v3/query-syntax#searchable-fields'>Searchable Fields</a>.

        search_engine : typing.Optional[SearchEngineVersionsEnum]
            The version of the search engine

        primary_order : typing.Optional[bool]
            If true (default), results are returned in a deterministic order. If false, results may be returned in a non-deterministic order, which can enhance performance for complex queries targeting a small number of users. Set to false only when consistent ordering and pagination is not required.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[UserResponseSchema, ListUsersOffsetPaginatedResponseContent]
            Users successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            response = await client.users.list(
                page=1,
                per_page=1,
                include_totals=True,
                sort="sort",
                connection="connection",
                fields="fields",
                include_fields=True,
                q="q",
                search_engine="v1",
                primary_order=True,
            )
            async for item in response:
                yield item

            # alternatively, you can paginate page-by-page
            async for page in response.iter_pages():
                yield page


        asyncio.run(main())
        """
        return await self._raw_client.list(
            page=page,
            per_page=per_page,
            include_totals=include_totals,
            sort=sort,
            connection=connection,
            fields=fields,
            include_fields=include_fields,
            q=q,
            search_engine=search_engine,
            primary_order=primary_order,
            request_options=request_options,
        )

    async def create(
        self,
        *,
        connection: str,
        email: typing.Optional[str] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        user_metadata: typing.Optional[UserMetadata] = OMIT,
        blocked: typing.Optional[bool] = OMIT,
        email_verified: typing.Optional[bool] = OMIT,
        phone_verified: typing.Optional[bool] = OMIT,
        app_metadata: typing.Optional[AppMetadata] = OMIT,
        given_name: typing.Optional[str] = OMIT,
        family_name: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        nickname: typing.Optional[str] = OMIT,
        picture: typing.Optional[str] = OMIT,
        user_id: typing.Optional[str] = OMIT,
        password: typing.Optional[str] = OMIT,
        verify_email: typing.Optional[bool] = OMIT,
        username: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateUserResponseContent:
        """
        Create a new user for a given [database](https://auth0.com/docs/connections/database) or [passwordless](https://auth0.com/docs/connections/passwordless) connection.

        Note: `connection` is required but other parameters such as `email` and `password` are dependent upon the type of connection.

        Parameters
        ----------
        connection : str
            Name of the connection this user should be created in.

        email : typing.Optional[str]
            The user's email.

        phone_number : typing.Optional[str]
            The user's phone number (following the E.164 recommendation).

        user_metadata : typing.Optional[UserMetadata]

        blocked : typing.Optional[bool]
            Whether this user was blocked by an administrator (true) or not (false).

        email_verified : typing.Optional[bool]
            Whether this email address is verified (true) or unverified (false). User will receive a verification email after creation if `email_verified` is false or not specified

        phone_verified : typing.Optional[bool]
            Whether this phone number has been verified (true) or not (false).

        app_metadata : typing.Optional[AppMetadata]

        given_name : typing.Optional[str]
            The user's given name(s).

        family_name : typing.Optional[str]
            The user's family name(s).

        name : typing.Optional[str]
            The user's full name.

        nickname : typing.Optional[str]
            The user's nickname.

        picture : typing.Optional[str]
            A URI pointing to the user's picture.

        user_id : typing.Optional[str]
            The external user's id provided by the identity provider.

        password : typing.Optional[str]
            Initial password for this user. Only valid for auth0 connection strategy.

        verify_email : typing.Optional[bool]
            Whether the user will receive a verification email after creation (true) or no email (false). Overrides behavior of `email_verified` parameter.

        username : typing.Optional[str]
            The user's username. Only valid if the connection requires a username.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateUserResponseContent
            User successfully created.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.create(
                connection="connection",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            connection=connection,
            email=email,
            phone_number=phone_number,
            user_metadata=user_metadata,
            blocked=blocked,
            email_verified=email_verified,
            phone_verified=phone_verified,
            app_metadata=app_metadata,
            given_name=given_name,
            family_name=family_name,
            name=name,
            nickname=nickname,
            picture=picture,
            user_id=user_id,
            password=password,
            verify_email=verify_email,
            username=username,
            request_options=request_options,
        )
        return _response.data

    async def list_users_by_email(
        self,
        *,
        email: str,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.List[UserResponseSchema]:
        """
        Find users by email. If Auth0 is the identity provider (idP), the email address associated with a user is saved in lower case, regardless of how you initially provided it.

        For example, if you register a user as JohnSmith@example.com, Auth0 saves the user's email as johnsmith@example.com.

        Therefore, when using this endpoint, make sure that you are searching for users via email addresses using the correct case.

        Parameters
        ----------
        email : str
            Email address to search for (case-sensitive).

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false). Defaults to true.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        typing.List[UserResponseSchema]
            Users successfully searched.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.list_users_by_email(
                fields="fields",
                include_fields=True,
                email="email",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_users_by_email(
            email=email, fields=fields, include_fields=include_fields, request_options=request_options
        )
        return _response.data

    async def get(
        self,
        id: str,
        *,
        fields: typing.Optional[str] = None,
        include_fields: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> GetUserResponseContent:
        """
        Retrieve user details. A list of fields to include or exclude may also be specified. For more information, see [Retrieve Users with the Get Users Endpoint](https://auth0.com/docs/manage-users/user-search/retrieve-users-with-get-users-endpoint).

        Parameters
        ----------
        id : str
            ID of the user to retrieve.

        fields : typing.Optional[str]
            Comma-separated list of fields to include or exclude (based on value provided for include_fields) in the result. Leave empty to retrieve all fields.

        include_fields : typing.Optional[bool]
            Whether specified fields are to be included (true) or excluded (false).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetUserResponseContent
            User successfully retrieved.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.get(
                id="id",
                fields="fields",
                include_fields=True,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(
            id, fields=fields, include_fields=include_fields, request_options=request_options
        )
        return _response.data

    async def delete(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> None:
        """
        Delete a user by user ID. This action cannot be undone. For Auth0 Dashboard instructions, see [Delete Users](https://auth0.com/docs/manage-users/user-accounts/delete-users).

        Parameters
        ----------
        id : str
            ID of the user to delete.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.delete(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(id, request_options=request_options)
        return _response.data

    async def update(
        self,
        id: str,
        *,
        blocked: typing.Optional[bool] = OMIT,
        email_verified: typing.Optional[bool] = OMIT,
        email: typing.Optional[str] = OMIT,
        phone_number: typing.Optional[str] = OMIT,
        phone_verified: typing.Optional[bool] = OMIT,
        user_metadata: typing.Optional[UserMetadata] = OMIT,
        app_metadata: typing.Optional[AppMetadata] = OMIT,
        given_name: typing.Optional[str] = OMIT,
        family_name: typing.Optional[str] = OMIT,
        name: typing.Optional[str] = OMIT,
        nickname: typing.Optional[str] = OMIT,
        picture: typing.Optional[str] = OMIT,
        verify_email: typing.Optional[bool] = OMIT,
        verify_phone_number: typing.Optional[bool] = OMIT,
        password: typing.Optional[str] = OMIT,
        connection: typing.Optional[str] = OMIT,
        client_id: typing.Optional[str] = OMIT,
        username: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> UpdateUserResponseContent:
        """
        Update a user.

        These are the attributes that can be updated at the root level:

        - app_metadata
        - blocked
        - email
        - email_verified
        - family_name
        - given_name
        - name
        - nickname
        - password
        - phone_number
        - phone_verified
        - picture
        - username
        - user_metadata
        - verify_email

        Some considerations:

        - The properties of the new object will replace the old ones.
        - The metadata fields are an exception to this rule (`user_metadata` and `app_metadata`). These properties are merged instead of being replaced but be careful, the merge only occurs on the first level.
        - If you are updating `email`, `email_verified`, `phone_number`, `phone_verified`, `username` or `password` of a secondary identity, you need to specify the `connection` property too.
        - If you are updating `email` or `phone_number` you can specify, optionally, the `client_id` property.
        - Updating `email_verified` is not supported for enterprise and passwordless sms connections.
        - Updating the `blocked` to `false` does not affect the user's blocked state from an excessive amount of incorrectly provided credentials. Use the "Unblock a user" endpoint from the "User Blocks" API to change the user's state.
        - Supported attributes can be unset by supplying `null` as the value.

        **Updating a field (non-metadata property)**

        To mark the email address of a user as verified, the body to send should be:

        ```json
        { "email_verified": true }
        ```

        **Updating a user metadata root property**

        Let's assume that our test user has the following `user_metadata`:

        ```json
        { "user_metadata" : { "profileCode": 1479 } }
        ```

        To add the field `addresses` the body to send should be:

        ```json
        { "user_metadata" : { "addresses": {"work_address": "100 Industrial Way"} }}
        ```

        The modified object ends up with the following `user_metadata` property:

        ```json
        {
          "user_metadata": {
            "profileCode": 1479,
            "addresses": { "work_address": "100 Industrial Way" }
          }
        }
        ```

        **Updating an inner user metadata property**

        If there's existing user metadata to which we want to add  `"home_address": "742 Evergreen Terrace"` (using the `addresses` property) we should send the whole `addresses` object. Since this is a first-level object, the object will be merged in, but its own properties will not be. The body to send should be:

        ```json
        {
          "user_metadata": {
            "addresses": {
              "work_address": "100 Industrial Way",
              "home_address": "742 Evergreen Terrace"
            }
          }
        }
        ```

        The modified object ends up with the following `user_metadata` property:

        ```json
        {
          "user_metadata": {
            "profileCode": 1479,
            "addresses": {
              "work_address": "100 Industrial Way",
              "home_address": "742 Evergreen Terrace"
            }
          }
        }
        ```

        Parameters
        ----------
        id : str
            ID of the user to update.

        blocked : typing.Optional[bool]
            Whether this user was blocked by an administrator (true) or not (false).

        email_verified : typing.Optional[bool]
            Whether this email address is verified (true) or unverified (false). If set to false the user will not receive a verification email unless `verify_email` is set to true.

        email : typing.Optional[str]
            Email address of this user.

        phone_number : typing.Optional[str]
            The user's phone number (following the E.164 recommendation).

        phone_verified : typing.Optional[bool]
            Whether this phone number has been verified (true) or not (false).

        user_metadata : typing.Optional[UserMetadata]
            User metadata to which this user has read/write access.

        app_metadata : typing.Optional[AppMetadata]
            User metadata to which this user has read-only access.

        given_name : typing.Optional[str]
            Given name/first name/forename of this user.

        family_name : typing.Optional[str]
            Family name/last name/surname of this user.

        name : typing.Optional[str]
            Name of this user.

        nickname : typing.Optional[str]
            Preferred nickname or alias of this user.

        picture : typing.Optional[str]
            URL to picture, photo, or avatar of this user.

        verify_email : typing.Optional[bool]
            Whether this user will receive a verification email after creation (true) or no email (false). Overrides behavior of `email_verified` parameter.

        verify_phone_number : typing.Optional[bool]
            Whether this user will receive a text after changing the phone number (true) or no text (false). Only valid when changing phone number for SMS connections.

        password : typing.Optional[str]
            New password for this user. Only valid for database connections.

        connection : typing.Optional[str]
            Name of the connection to target for this user update.

        client_id : typing.Optional[str]
            Auth0 client ID. Only valid when updating email address.

        username : typing.Optional[str]
            The user's username. Only valid if the connection requires a username.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        UpdateUserResponseContent
            User successfully updated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.update(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update(
            id,
            blocked=blocked,
            email_verified=email_verified,
            email=email,
            phone_number=phone_number,
            phone_verified=phone_verified,
            user_metadata=user_metadata,
            app_metadata=app_metadata,
            given_name=given_name,
            family_name=family_name,
            name=name,
            nickname=nickname,
            picture=picture,
            verify_email=verify_email,
            verify_phone_number=verify_phone_number,
            password=password,
            connection=connection,
            client_id=client_id,
            username=username,
            request_options=request_options,
        )
        return _response.data

    async def regenerate_recovery_code(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> RegenerateUsersRecoveryCodeResponseContent:
        """
        Remove an existing multi-factor authentication (MFA) [recovery code](https://auth0.com/docs/secure/multi-factor-authentication/reset-user-mfa) and generate a new one. If a user cannot access the original device or account used for MFA enrollment, they can use a recovery code to authenticate.

        Parameters
        ----------
        id : str
            ID of the user to regenerate a multi-factor authentication recovery code for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        RegenerateUsersRecoveryCodeResponseContent
            New recovery code successfully generated.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.regenerate_recovery_code(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.regenerate_recovery_code(id, request_options=request_options)
        return _response.data

    async def revoke_access(
        self,
        id: str,
        *,
        session_id: typing.Optional[str] = OMIT,
        preserve_refresh_tokens: typing.Optional[bool] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> None:
        """
        Revokes selected resources related to a user (sessions, refresh tokens, ...).

        Parameters
        ----------
        id : str
            ID of the user.

        session_id : typing.Optional[str]
            ID of the session to revoke.

        preserve_refresh_tokens : typing.Optional[bool]
            Whether to preserve the refresh tokens associated with the session.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        None

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.users.revoke_access(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.revoke_access(
            id, session_id=session_id, preserve_refresh_tokens=preserve_refresh_tokens, request_options=request_options
        )
        return _response.data

    @property
    def authentication_methods(self):
        if self._authentication_methods is None:
            from .authentication_methods.client import AsyncAuthenticationMethodsClient  # noqa: E402

            self._authentication_methods = AsyncAuthenticationMethodsClient(client_wrapper=self._client_wrapper)
        return self._authentication_methods

    @property
    def authenticators(self):
        if self._authenticators is None:
            from .authenticators.client import AsyncAuthenticatorsClient  # noqa: E402

            self._authenticators = AsyncAuthenticatorsClient(client_wrapper=self._client_wrapper)
        return self._authenticators

    @property
    def connected_accounts(self):
        if self._connected_accounts is None:
            from .connected_accounts.client import AsyncConnectedAccountsClient  # noqa: E402

            self._connected_accounts = AsyncConnectedAccountsClient(client_wrapper=self._client_wrapper)
        return self._connected_accounts

    @property
    def effective_permissions(self):
        if self._effective_permissions is None:
            from .effective_permissions.client import AsyncEffectivePermissionsClient  # noqa: E402

            self._effective_permissions = AsyncEffectivePermissionsClient(client_wrapper=self._client_wrapper)
        return self._effective_permissions

    @property
    def effective_roles(self):
        if self._effective_roles is None:
            from .effective_roles.client import AsyncEffectiveRolesClient  # noqa: E402

            self._effective_roles = AsyncEffectiveRolesClient(client_wrapper=self._client_wrapper)
        return self._effective_roles

    @property
    def enrollments(self):
        if self._enrollments is None:
            from .enrollments.client import AsyncEnrollmentsClient  # noqa: E402

            self._enrollments = AsyncEnrollmentsClient(client_wrapper=self._client_wrapper)
        return self._enrollments

    @property
    def groups(self):
        if self._groups is None:
            from .groups.client import AsyncGroupsClient  # noqa: E402

            self._groups = AsyncGroupsClient(client_wrapper=self._client_wrapper)
        return self._groups

    @property
    def identities(self):
        if self._identities is None:
            from .identities.client import AsyncIdentitiesClient  # noqa: E402

            self._identities = AsyncIdentitiesClient(client_wrapper=self._client_wrapper)
        return self._identities

    @property
    def logs(self):
        if self._logs is None:
            from .logs.client import AsyncLogsClient  # noqa: E402

            self._logs = AsyncLogsClient(client_wrapper=self._client_wrapper)
        return self._logs

    @property
    def multifactor(self):
        if self._multifactor is None:
            from .multifactor.client import AsyncMultifactorClient  # noqa: E402

            self._multifactor = AsyncMultifactorClient(client_wrapper=self._client_wrapper)
        return self._multifactor

    @property
    def organizations(self):
        if self._organizations is None:
            from .organizations.client import AsyncOrganizationsClient  # noqa: E402

            self._organizations = AsyncOrganizationsClient(client_wrapper=self._client_wrapper)
        return self._organizations

    @property
    def permissions(self):
        if self._permissions is None:
            from .permissions.client import AsyncPermissionsClient  # noqa: E402

            self._permissions = AsyncPermissionsClient(client_wrapper=self._client_wrapper)
        return self._permissions

    @property
    def risk_assessments(self):
        if self._risk_assessments is None:
            from .risk_assessments.client import AsyncRiskAssessmentsClient  # noqa: E402

            self._risk_assessments = AsyncRiskAssessmentsClient(client_wrapper=self._client_wrapper)
        return self._risk_assessments

    @property
    def roles(self):
        if self._roles is None:
            from .roles.client import AsyncRolesClient  # noqa: E402

            self._roles = AsyncRolesClient(client_wrapper=self._client_wrapper)
        return self._roles

    @property
    def refresh_token(self):
        if self._refresh_token is None:
            from .refresh_token.client import AsyncRefreshTokenClient  # noqa: E402

            self._refresh_token = AsyncRefreshTokenClient(client_wrapper=self._client_wrapper)
        return self._refresh_token

    @property
    def sessions(self):
        if self._sessions is None:
            from .sessions.client import AsyncSessionsClient  # noqa: E402

            self._sessions = AsyncSessionsClient(client_wrapper=self._client_wrapper)
        return self._sessions



# This file was auto-generated by Fern from our API Definition.

import typing

ClientTokenEndpointAuthMethodOrNullEnum = typing.Union[
    typing.Literal["none", "client_secret_post", "client_secret_basic"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2
from ..core.serialization import FieldMetadata
from .connection_domain_aliases import ConnectionDomainAliases
from .connection_icon_url_adfs import ConnectionIconUrlAdfs
from .connection_metadata_xml_adfs import ConnectionMetadataXmlAdfs
from .connection_options_common import ConnectionOptionsCommon
from .connection_set_user_root_attributes_enum import ConnectionSetUserRootAttributesEnum
from .connection_should_trust_email_verified_connection_enum import ConnectionShouldTrustEmailVerifiedConnectionEnum
from .connection_sign_in_endpoint_adfs import ConnectionSignInEndpointAdfs
from .connection_tenant_domain import ConnectionTenantDomain
from .connection_thumbprints import ConnectionThumbprints
from .connection_upstream_params import ConnectionUpstreamParams


class ConnectionOptionsAdfs(ConnectionOptionsCommon):
    """
    Options for the 'adfs' connection
    """

    adfs_server: typing.Optional[str] = pydantic.Field(default=None)
    """
    ADFS federation metadata host or XML URL used to discover WS-Fed endpoints and certificates. Errors if adfs_server and fedMetadataXml are both absent.
    """

    domain_aliases: typing.Optional[ConnectionDomainAliases] = None
    entity_id: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="entityId"),
        pydantic.Field(
            alias="entityId",
            description="The entity identifier (Issuer) for the ADFS Service Provider. When not provided, defaults to 'urn:auth0:{tenant}:{connection}'.",
        ),
    ] = None
    """
    The entity identifier (Issuer) for the ADFS Service Provider. When not provided, defaults to 'urn:auth0:{tenant}:{connection}'.
    """

    fed_metadata_xml: typing_extensions.Annotated[
        typing.Optional[ConnectionMetadataXmlAdfs],
        FieldMetadata(alias="fedMetadataXml"),
        pydantic.Field(alias="fedMetadataXml"),
    ] = None
    icon_url: typing.Optional[ConnectionIconUrlAdfs] = None
    prev_thumbprints: typing.Optional[ConnectionThumbprints] = None
    set_user_root_attributes: typing.Optional[ConnectionSetUserRootAttributesEnum] = None
    should_trust_email_verified_connection: typing.Optional[ConnectionShouldTrustEmailVerifiedConnectionEnum] = None
    sign_in_endpoint: typing_extensions.Annotated[
        typing.Optional[ConnectionSignInEndpointAdfs],
        FieldMetadata(alias="signInEndpoint"),
        pydantic.Field(alias="signInEndpoint"),
    ] = None
    tenant_domain: typing.Optional[ConnectionTenantDomain] = None
    thumbprints: typing.Optional[ConnectionThumbprints] = None
    upstream_params: typing.Optional[ConnectionUpstreamParams] = None
    user_id_attribute: typing.Optional[str] = pydantic.Field(default=None)
    """
    Custom ADFS claim to use as the unique user identifier. When provided, this attribute is prepended to the default user_id mapping list with highest priority. Accepts a string (single ADFS claim name).
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from .flow_action_http_send_request_params_query_params_value import FlowActionHttpSendRequestParamsQueryParamsValue

FlowActionHttpSendRequestParamsQueryParams = typing.Dict[
    str, typing.Optional[FlowActionHttpSendRequestParamsQueryParamsValue]
]



# This file was auto-generated by Fern from our API Definition.

import typing

DomainCertificateStatusEnum = typing.Union[
    typing.Literal["provisioning", "provisioning_failed", "provisioned", "renewing_failed"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .form_block_resend_button_config import FormBlockResendButtonConfig
from .form_block_type_resend_button_const import FormBlockTypeResendButtonConst
from .form_component_category_block_const import FormComponentCategoryBlockConst


class FormBlockResendButton(UniversalBaseModel):
    id: str
    category: FormComponentCategoryBlockConst
    type: FormBlockTypeResendButtonConst
    config: FormBlockResendButtonConfig

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.request_options import RequestOptions
from ...types.create_export_users_fields import CreateExportUsersFields
from ...types.create_export_users_response_content import CreateExportUsersResponseContent
from ...types.job_file_format_enum import JobFileFormatEnum
from .raw_client import AsyncRawUsersExportsClient, RawUsersExportsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class UsersExportsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawUsersExportsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawUsersExportsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawUsersExportsClient
        """
        return self._raw_client

    def create(
        self,
        *,
        connection_id: typing.Optional[str] = OMIT,
        format: typing.Optional[JobFileFormatEnum] = OMIT,
        limit: typing.Optional[int] = OMIT,
        fields: typing.Optional[typing.Sequence[CreateExportUsersFields]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateExportUsersResponseContent:
        """
        Export all users to a file via a long-running job.

        Parameters
        ----------
        connection_id : typing.Optional[str]
            connection_id of the connection from which users will be exported.

        format : typing.Optional[JobFileFormatEnum]

        limit : typing.Optional[int]
            Limit the number of records.

        fields : typing.Optional[typing.Sequence[CreateExportUsersFields]]
            List of fields to be included in the CSV. Defaults to a predefined set of fields.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateExportUsersResponseContent
            Job created successfully.

        Examples
        --------
        from auth0 import Auth0

        client = Auth0(
            token="YOUR_TOKEN",
        )
        client.jobs.users_exports.create()
        """
        _response = self._raw_client.create(
            connection_id=connection_id, format=format, limit=limit, fields=fields, request_options=request_options
        )
        return _response.data


class AsyncUsersExportsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawUsersExportsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawUsersExportsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawUsersExportsClient
        """
        return self._raw_client

    async def create(
        self,
        *,
        connection_id: typing.Optional[str] = OMIT,
        format: typing.Optional[JobFileFormatEnum] = OMIT,
        limit: typing.Optional[int] = OMIT,
        fields: typing.Optional[typing.Sequence[CreateExportUsersFields]] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CreateExportUsersResponseContent:
        """
        Export all users to a file via a long-running job.

        Parameters
        ----------
        connection_id : typing.Optional[str]
            connection_id of the connection from which users will be exported.

        format : typing.Optional[JobFileFormatEnum]

        limit : typing.Optional[int]
            Limit the number of records.

        fields : typing.Optional[typing.Sequence[CreateExportUsersFields]]
            List of fields to be included in the CSV. Defaults to a predefined set of fields.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CreateExportUsersResponseContent
            Job created successfully.

        Examples
        --------
        import asyncio

        from auth0 import AsyncAuth0

        client = AsyncAuth0(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.jobs.users_exports.create()


        asyncio.run(main())
        """
        _response = await self._raw_client.create(
            connection_id=connection_id, format=format, limit=limit, fields=fields, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

ConnectionUserinfoEncryptionEncValuesSupported = typing.List[str]
"""
JSON array containing a list of the JWE encryption algorithms (enc values) [JWA] supported by the UserInfo Endpoint to encode the Claims in a JWT [JWT].
"""



from .conftest import get_client, verify_request_count


def test_riskAssessments_settings_get() -> None:
    """Test get endpoint with WireMock"""
    test_id = "risk_assessments.settings.get.0"
    client = get_client(test_id)
    client.risk_assessments.settings.get()
    verify_request_count(test_id, "GET", "/risk-assessments/settings", None, 1)


def test_riskAssessments_settings_update() -> None:
    """Test update endpoint with WireMock"""
    test_id = "risk_assessments.settings.update.0"
    client = get_client(test_id)
    client.risk_assessments.settings.update(
        enabled=True,
    )
    verify_request_count(test_id, "PATCH", "/risk-assessments/settings", None, 1)



from .conftest import get_client, verify_request_count


def test_stats_get_active_users_count() -> None:
    """Test getActiveUsersCount endpoint with WireMock"""
    test_id = "stats.get_active_users_count.0"
    client = get_client(test_id)
    client.stats.get_active_users_count()
    verify_request_count(test_id, "GET", "/stats/active-users", None, 1)


def test_stats_get_daily() -> None:
    """Test getDaily endpoint with WireMock"""
    test_id = "stats.get_daily.0"
    client = get_client(test_id)
    client.stats.get_daily(
        from_="from",
        to="to",
    )
    verify_request_count(test_id, "GET", "/stats/daily", {"from": "from", "to": "to"}, 1)



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.jsonable_encoder import encode_path_param
from ...core.pagination import AsyncPager, SyncPager
from ...core.parse_error import ParsingError
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.too_many_requests_error import TooManyRequestsError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.log import Log
from ...types.user_list_log_offset_paginated_response_content import UserListLogOffsetPaginatedResponseContent
from pydantic import ValidationError


class RawLogsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        sort: typing.Optional[str] = None,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SyncPager[Log, UserListLogOffsetPaginatedResponseContent]:
        """
        Retrieve log events for a specific user.

        Note: For more information on all possible event types, their respective acronyms and descriptions, see <a href="https://auth0.com/docs/logs/log-event-type-codes">Log Event Type Codes</a>.

        For more information on the list of fields that can be used in `sort`, see <a href="https://auth0.com/docs/logs/log-search-query-syntax#searchable-fields">Searchable Fields</a>.

        Auth0 <a href="https://auth0.com/docs/logs/retrieve-log-events-using-mgmt-api#limitations">limits the number of logs</a> you can return by search criteria to 100 logs per request. Furthermore, you may only paginate through up to 1,000 search results. If you exceed this threshold, please redefine your search.

        Parameters
        ----------
        id : str
            ID of the user of the logs to retrieve

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Paging is disabled if parameter not sent.

        sort : typing.Optional[str]
            Field to sort by. Use `fieldname:1` for ascending order and `fieldname:-1` for descending.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SyncPager[Log, UserListLogOffsetPaginatedResponseContent]
            Logs successfully retrieved.
        """
        page = page if page is not None else 0

        _response = self._client_wrapper.httpx_client.request(
            f"users/{encode_path_param(id)}/logs",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
                "sort": sort,
                "include_totals": include_totals,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    UserListLogOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=UserListLogOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.logs
                _has_next = len(_items or []) > 0
                _get_next = lambda: self.list(
                    id,
                    page=page + 1,
                    per_page=per_page,
                    sort=sort,
                    include_totals=include_totals,
                    request_options=request_options,
                )
                return SyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawLogsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        id: str,
        *,
        page: typing.Optional[int] = 0,
        per_page: typing.Optional[int] = 50,
        sort: typing.Optional[str] = None,
        include_totals: typing.Optional[bool] = True,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncPager[Log, UserListLogOffsetPaginatedResponseContent]:
        """
        Retrieve log events for a specific user.

        Note: For more information on all possible event types, their respective acronyms and descriptions, see <a href="https://auth0.com/docs/logs/log-event-type-codes">Log Event Type Codes</a>.

        For more information on the list of fields that can be used in `sort`, see <a href="https://auth0.com/docs/logs/log-search-query-syntax#searchable-fields">Searchable Fields</a>.

        Auth0 <a href="https://auth0.com/docs/logs/retrieve-log-events-using-mgmt-api#limitations">limits the number of logs</a> you can return by search criteria to 100 logs per request. Furthermore, you may only paginate through up to 1,000 search results. If you exceed this threshold, please redefine your search.

        Parameters
        ----------
        id : str
            ID of the user of the logs to retrieve

        page : typing.Optional[int]
            Page index of the results to return. First page is 0.

        per_page : typing.Optional[int]
            Number of results per page. Paging is disabled if parameter not sent.

        sort : typing.Optional[str]
            Field to sort by. Use `fieldname:1` for ascending order and `fieldname:-1` for descending.

        include_totals : typing.Optional[bool]
            Return results inside an object that contains the total result count (true) or as a direct array of results (false, default).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncPager[Log, UserListLogOffsetPaginatedResponseContent]
            Logs successfully retrieved.
        """
        page = page if page is not None else 0

        _response = await self._client_wrapper.httpx_client.request(
            f"users/{encode_path_param(id)}/logs",
            method="GET",
            params={
                "page": page,
                "per_page": per_page,
                "sort": sort,
                "include_totals": include_totals,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _parsed_response = typing.cast(
                    UserListLogOffsetPaginatedResponseContent,
                    parse_obj_as(
                        type_=UserListLogOffsetPaginatedResponseContent,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                _items = _parsed_response.logs
                _has_next = len(_items or []) > 0

                async def _get_next():
                    return await self.list(
                        id,
                        page=page + 1,
                        per_page=per_page,
                        sort=sort,
                        include_totals=include_totals,
                        request_options=request_options,
                    )

                return AsyncPager(has_next=_has_next, items=_items, get_next=_get_next, response=_parsed_response)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 429:
                raise TooManyRequestsError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Any,
                        parse_obj_as(
                            type_=typing.Any,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        except ValidationError as e:
            raise ParsingError(
                status_code=_response.status_code, headers=dict(_response.headers), body=_response.json(), cause=e
            )
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing

CreateConnectionRequestContentSalesforceCommunityStrategy = typing.Union[
    typing.Literal["salesforce-community"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

BreachedPasswordDetectionPreChangePasswordShieldsEnum = typing.Union[
    typing.Literal["block", "admin_notification"], typing.Any
]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class ClientAddonAzureBlob(UniversalBaseModel):
    """
    Azure Blob Storage addon configuration.
    """

    account_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="accountName"),
        pydantic.Field(
            alias="accountName",
            description="Your Azure storage account name. Usually first segment in your Azure storage URL. e.g. `https://acme-org.blob.core.windows.net` would be the account name `acme-org`.",
        ),
    ] = None
    """
    Your Azure storage account name. Usually first segment in your Azure storage URL. e.g. `https://acme-org.blob.core.windows.net` would be the account name `acme-org`.
    """

    storage_access_key: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="storageAccessKey"),
        pydantic.Field(alias="storageAccessKey", description="Access key associated with this storage account."),
    ] = None
    """
    Access key associated with this storage account.
    """

    container_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="containerName"),
        pydantic.Field(alias="containerName", description="Container to request a token for. e.g. `my-container`."),
    ] = None
    """
    Container to request a token for. e.g. `my-container`.
    """

    blob_name: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="blobName"),
        pydantic.Field(
            alias="blobName",
            description="Entity to request a token for. e.g. `my-blob`. If blank the computed SAS will apply to the entire storage container.",
        ),
    ] = None
    """
    Entity to request a token for. e.g. `my-blob`. If blank the computed SAS will apply to the entire storage container.
    """

    expiration: typing.Optional[int] = pydantic.Field(default=None)
    """
    Expiration in minutes for the generated token (default of 5 minutes).
    """

    signed_identifier: typing_extensions.Annotated[
        typing.Optional[str],
        FieldMetadata(alias="signedIdentifier"),
        pydantic.Field(
            alias="signedIdentifier",
            description="Shared access policy identifier defined in your storage account resource.",
        ),
    ] = None
    """
    Shared access policy identifier defined in your storage account resource.
    """

    blob_read: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if the issued token has permission to read the content, properties, metadata and block list. Use the blob as the source of a copy operation.
    """

    blob_write: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if the issued token has permission to create or write content, properties, metadata, or block list. Snapshot or lease the blob. Resize the blob (page blob only). Use the blob as the destination of a copy operation within the same account.
    """

    blob_delete: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if the issued token has permission to delete the blob.
    """

    container_read: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if the issued token has permission to read the content, properties, metadata or block list of any blob in the container. Use any blob in the container as the source of a copy operation
    """

    container_write: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates that for any blob in the container if the issued token has permission to create or write content, properties, metadata, or block list. Snapshot or lease the blob. Resize the blob (page blob only). Use the blob as the destination of a copy operation within the same account.
    """

    container_delete: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if issued token has permission to delete any blob in the container.
    """

    container_list: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Indicates if the issued token has permission to list blobs in the container.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class ClientMobileAndroid(UniversalBaseModel):
    """
    Android native app configuration.
    """

    app_package_name: typing.Optional[str] = pydantic.Field(default=None)
    """
    App package name found in AndroidManifest.xml.
    """

    sha_256_cert_fingerprints: typing_extensions.Annotated[
        typing.Optional[typing.List[str]],
        FieldMetadata(alias="sha256_cert_fingerprints"),
        pydantic.Field(
            alias="sha256_cert_fingerprints",
            description="SHA256 fingerprints of the app's signing certificate. Multiple fingerprints can be used to support different versions of your app, such as debug and production builds.",
        ),
    ] = None
    """
    SHA256 fingerprints of the app's signing certificate. Multiple fingerprints can be used to support different versions of your app, such as debug and production builds.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .form_block_resend_button_config_text_alignment_enum import FormBlockResendButtonConfigTextAlignmentEnum


class FormBlockResendButtonConfig(UniversalBaseModel):
    active_text: str
    button_text: str
    waiting_text: str
    text_alignment: typing.Optional[FormBlockResendButtonConfigTextAlignmentEnum] = None
    flow_id: str
    max_attempts: typing.Optional[float] = None
    waiting_time: typing.Optional[float] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from .connection_api_enable_groups import ConnectionApiEnableGroups

ConnectionApiEnableGroupsGoogleApps = ConnectionApiEnableGroups
"""
Enables integration with the Google Workspace Admin SDK Directory API for groups. When true, Auth0 can synchronize groups & group memberships and supports inbound directory provisioning for groups. Defaults to false.
"""



# This file was auto-generated by Fern from our API Definition.

from .connection_totp_time_step_passwordless import ConnectionTotpTimeStepPasswordless

ConnectionTotpTimeStepEmail = ConnectionTotpTimeStepPasswordless
"""
Time step for TOTP in seconds
"""
